package demo.util;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.ServiceLoader;

public final class GenericServiceLocator {

    private GenericServiceLocator() {
    }

    public static <T> T locate(final Class<T> clazz) {
        final List<T> services = locateAll(clazz);
        return services.isEmpty() ? (T) null : services.get(0);
    }

    public static <T> List<T> locateAll(final Class <T> clazz) {

        final Iterator <T> iterator = ServiceLoader.load(clazz).iterator();
        final List<T> services = new ArrayList<T>();

        while(iterator.hasNext()) {
            try {
                services.add(iterator.next());
            } catch (Error e) {
                e.printStackTrace(System.err);
            }
        }

        return services;

    }
}
